@@ -9,8 +9,8 @@ import queue
 import signal
 import sys
 import serial
-from time import sleep
 import os
+import enum
 
 class SerialPortManager(threading.Thread):
 
@@ -29,6 +29,7 @@ class SerialPortManager(threading.Thread):
             self.intQueue = intQueue
             self.stopper = stopper
             self.exception = None
+            self.daemon = True
 
         def run(self):
 
@@ -112,6 +113,18 @@ class SignalHandler:
 
         self.manager.join()
 
+
+class binhoException(Exception):
+    pass
+
+
+class oneWireCmd(enum.Enum):
+    """Enum for exchangeBytes1WIRE"""
+    NONE = "NONE"
+    SELECT = "SELECT"
+    SKIP = "SKIP"
+
+
 class binhoHostAdapter:
 
     # Constructor, needs a serial port as a parameter
@@ -193,6 +206,14 @@ class binhoHostAdapter:
         while self._intQueue.empty() == False:
             self.interrupts.add(self._intQueue.get())
 
+    def _checkDeviceSuccess(self, ret_str):
+        if ret_str == '-OK':
+            return True
+        elif ret_str == '-NG':
+            return False
+        else:
+            raise binhoException(f'Invalid command response: {ret_str}')
+
     # Public functions
 
     # Communication Management
@@ -564,6 +585,76 @@ class binhoHostAdapter:
 
         self.interruptClear('!I2C' + str(i2cIndex) + ' SLAVE RX')
 
+    def setSlaveRegisterI2C(self, i2cIndex, register, value):
+
+        self._sendCommand('I2C' + str(i2cIndex) + ' SLAVE REG ' + str(register) + ' ' + str(value))
+        result = self._readResponse()
+
+        return result
+
+    def getSlaveRegisterI2C(self, i2cIndex, register):
+
+        self._sendCommand('I2C' + str(i2cIndex) + ' SLAVE REG ' + str(register) + ' ?')
+        result = self._readResponse()
+
+        return result
+
+    def setSlaveReadMaskI2C(self, i2cIndex, register, value):
+
+        self._sendCommand('I2C' + str(i2cIndex) + ' SLAVE READMASK ' + str(register) + ' ' + str(value))
+        result = self._readResponse()
+
+        return result
+
+    def getSlaveReadMaskI2C(self, i2cIndex, register):
+
+        self._sendCommand('I2C' + str(i2cIndex) + ' SLAVE READMASK ' + str(register) + ' ?')
+        result = self._readResponse()
+
+        return result
+
+    def setSlaveWriteMaskI2C(self, i2cIndex, register, value):
+
+        self._sendCommand('I2C' + str(i2cIndex) + ' SLAVE WRITEMASK ' + str(register) + ' ' + str(value))
+        result = self._readResponse()
+
+        return result
+
+    def getSlaveWriteMaskI2C(self, i2cIndex, register):
+
+        self._sendCommand('I2C' + str(i2cIndex) + ' SLAVE WRITEMASK ' + str(register) + ' ?')
+        result = self._readResponse()
+
+        return result
+
+    def setSlaveModeI2C(self, i2cIndex, mode):
+
+        self._sendCommand('I2C' + str(i2cIndex) + ' SLAVE MODE ' + str(mode))
+        result = self._readResponse()
+
+        return result
+
+    def getSlaveModeI2C(self, i2cIndex):
+
+        self._sendCommand('I2C' + str(i2cIndex) + ' SLAVE MODE ' + '?')
+        result = self._readResponse()
+
+        return result
+
+    def setSlaveRegisterCount(self, i2cIndex, registerCount):
+
+        self._sendCommand('I2C' + str(i2cIndex) + ' SLAVE REGCNT ' + str(registerCount))
+        result = self._readResponse()
+
+        return result
+
+    def getSlaveRegisterCount(self, i2cIndex):
+
+        self._sendCommand('I2C' + str(i2cIndex) + ' SLAVE REGCNT ' + '?')
+        result = self._readResponse()
+
+        return result
+
     ## SPI COMMANDS
 
     def setClockSPI(self, spiIndex, clock):
@@ -699,7 +790,7 @@ class binhoHostAdapter:
     def getBaudRateUART(self, uartIndex):
 
         self._sendCommand('UART' + str(uartIndex) + ' BAUD ?')
-        result - self._readResponse()
+        result = self._readResponse()
 
         return result
 
@@ -786,94 +877,257 @@ class binhoHostAdapter:
 
     ## 1WIRE COMMANDS
 
-    def begin1WIRE(self, oneWireIndex, pin, pullup):
+    def begin1WIRE(self, pin, pullup=True, oneWireIndex=0):
+        """
+        This function starts the 1-WIRE Master on the given IO pin. The 1-Wire protocol can be used
+        on any of the IO pins, however it is especially convenient to use it on IO0 and IO2 as the
+        internal pull-up resistor can be used thus eliminating the need for an external pull-up resistor.
 
-        if not pullup:
-            self._sendCommand('1WIRE' + str(oneWireIndex) + ' BEGIN ' + str(pin))
+        :param pin: The binho pin to use for 1-Wire comms can be 0-4
+        :type pin: int
+        :param pullup: True or False, enable the pullup on the IO pin
+        :type pullup: bool
+        :param oneWireIndex: The 1-Wire index, 0 on the binho nova
+        :type oneWireIndex: int
+        :raises binhoException: if the command is unsuccessful
+        :return: None
+        :rtype: None
+        """
+
+        if pullup:
+            self._sendCommand(f'1WIRE{oneWireIndex} BEGIN {pin} PULL')
         else:
-            self._sendCommand('1WIRE' + str(oneWireIndex) + ' BEGIN ' + str(pin) + ' PULL')
+            self._sendCommand(f'1WIRE{oneWireIndex} BEGIN {pin}')
 
-        result = self._readResponse()
+        if not self._checkDeviceSuccess(self._readResponse()):
+            raise binhoException('Error executing 1-Wire Begin received NAK')
 
-        return result
+    def reset1WIRE(self, oneWireIndex=0):
+        """
+        Send a 1-Wire reset pulse, return True if the binho receives a
+        presence pulse False otherwise
 
-    def reset1WIRE(self, oneWireIndex):
+        :param oneWireIndex: The 1-Wire index, 0 on the binho nova
+        :type oneWireIndex: int
+        :return: True if the binho receives a presence pulse False otherwise
+        :rtype: bool
+        """
 
-        self._sendCommand('1WIRE' + str(oneWireIndex) + ' RESET')
-        result = self._readResponse()
+        self._sendCommand(f'1WIRE{oneWireIndex} RESET')
 
-        return result
+        return self._checkDeviceSuccess(self._readResponse())
 
-    def writeByte1WIRE(self, oneWireIndex, data, powered = False):
+    def writeByte1WIRE(self, data, oneWireIndex=0, powered=True):
+        """
+        Write a byte over 1-Wire
 
-        if not powered:
-            self._sendCommand('1WIRE' + str(oneWireIndex) + ' WRITE ' + str(data))
+        :param data: Data byte to write
+        :type data: int
+        :param oneWireIndex: The 1-Wire index, 0 on the binho nova
+        :type oneWireIndex: int
+        :param powered: Leave 1-Wire power on after this command finsihes
+        :type powered: bool
+        :raises binhoException: if the command is unsuccessful
+        :return: None
+        :rtype: None
+        """
+
+        if not 0 <= data <= 255:
+            raise binhoException(f'Data byte must be in range 0-255, not {data}')
+
+        if powered:
+            self._sendCommand(f'1WIRE{oneWireIndex} WRITE {data} POWER')
         else:
-            self._sendCommand('1WIRE' + str(oneWireIndex) + ' WRITE ' + str(data) + ' POWER')
+            self._sendCommand(f'1WIRE{oneWireIndex} WRITE {data}')
+
+        if not self._checkDeviceSuccess(self._readResponse()):
+            raise binhoException('Error executing 1-Wire Write received NAK')
+
+    def readByte1WIRE(self, oneWireIndex=0):
+        """
+        Reads one byte over 1-Wire
+
+        :param oneWireIndex: The 1-Wire index, 0 on the binho nova
+        :type oneWireIndex: int
+        :raises binhoException: if the command is unsuccessful
+        :return: Received byte
+        :rtype: int
+        """
+        self._sendCommand(f'1WIRE{oneWireIndex} READ')
+        result = self._readResponse()
+
+        if result == '-NG':
+            raise binhoException('Error executing 1-Wire Read received NAK')
+
+
+        return int(result[15:], 16)
+
+    def exchangeBytes1WIRE(self, oneWireCmd=oneWireCmd.NONE, bytesToWrite=None, bytesToRead=0, oneWireIndex=0):
+        """
+        Issues one of the following commands to the one wire bus:
+            None (this is the default)
+            Reset then Select ROM (requires a one wire address in the address buffer)
+            Reset then Skip ROM
+        Then writes bytesToWrite (max 1024) out to the bus
+        Then reads bytesToRead (max 1024) from the bus
+
+        :param oneWireCmd: 1-Wire command to send out on the bus
+        :type oneWireCmd: oneWireCmd
+        :param bytesToWrite: bytearray of bytes to write out to the 1-Wire bus after the 1-Wire Cmd
+        :type bytesToWrite: bytearray
+        :param bytesToRead: Number of bytes to read after writeing bytes
+        :type bytesToRead: int
+        :param oneWireIndex: The 1-Wire index, 0 on the binho nova
+        :type oneWireIndex: int
+        :return: A bytearray of bytes that were read out
+        :rtype: bytearray
+        """
+
+        if len(bytesToWrite) > 1024:
+            raise ValueError("WHR command can only write 1024 bytea at a time!")
+
+        if bytesToRead > 1024:
+            raise ValueError("WHR command can only read 1024 bytea at a time!")
+
+        self._sendCommand(f'1WIRE{oneWireIndex} '
+                          f'WHR {oneWireCmd.value} '
+                          f'{bytesToRead} '
+                          f'{len(bytesToWrite)} '
+                          f'{"".join(f"{b:02x}" for b in bytesToWrite)}'
+                    )
 
         result = self._readResponse()
 
-        return result
+        if bytesToRead == 0:
+            if not result.startswith("-OK"):
+                raise RuntimeError(f"Error Binho responded with {result}, not the expected \"-OK\"")
+            else:
+                return bytearray()
+        else:
+            if not result.startswith("-1WIRE0 RXD "):
+                raise RuntimeError(f"Error Binho responded with {result}, not the expected \"-1WIRE0 RXD ...\"")
+            else:
+                return bytearray.fromhex(result[12:])
 
-    def readByte1WIRE(self, oneWireIndex):
+    def select1WIRE(self, oneWireIndex=0):
+        """
+        Select the device that was found as a result of the most recent SEARCH command
 
-        self._sendCommand('1WIRE' + str(oneWireIndex) + ' READ')
+        :param oneWireIndex: The 1-Wire index, 0 on the binho nova
+        :type oneWireIndex: int
+        :raises binhoException: if the command is unsuccessful
+        :return: None
+        :rtype: None
+        """
+        self._sendCommand(f'1WIRE{oneWireIndex} SELECT')
+
+        if not self._checkDeviceSuccess(self._readResponse()):
+            raise binhoException('Error executing 1-Wire Select received NAK')
+
+    def skip1WIRE(self, oneWireIndex=0):
+        """
+        Skip the device selection process, if there is one device on the bus
+
+        :param oneWireIndex: The 1-Wire index, 0 on the binho nova
+        :type oneWireIndex: int
+        :raises binhoException: if the command is unsuccessful
+        :return: None
+        :rtype: None
+        """
+        self._sendCommand(f'1WIRE{oneWireIndex} SKIP')
+
+        if not self._checkDeviceSuccess(self._readResponse()):
+            raise binhoException('Error executing 1-Wire Skip received NAK')
+
+    def depower1WIRE(self, oneWireIndex=0):
+        """
+        Power down the 1-WIRE bus
+
+        :param oneWireIndex: The 1-Wire index, 0 on the binho nova
+        :type oneWireIndex: int
+        :raises binhoException: if the command is unsuccessful
+        :return: None
+        :rtype: None
+        """
+        self._sendCommand(f'1WIRE{oneWireIndex} DEPOWER')
+
+        if not self._checkDeviceSuccess(self._readResponse()):
+            raise binhoException('Error executing 1-Wire depower received NAK')
+
+    def getAddress1WIRE(self, oneWireIndex=0):
+        """
+        Returns the address that was found by performing a 1-Wire search
+
+        :param oneWireIndex: The 1-Wire index, 0 on the binho nova
+        :type oneWireIndex: int
+        :raises binhoException: if the command is unsuccessful
+        :return: Bytearray of the address
+        :rtype: bytearray
+        """
+        self._sendCommand(f'1WIRE{oneWireIndex} ADDR ?')
         result = self._readResponse()
 
-        return result
+        if result == '-NG':
+            raise binhoException('Error executing 1-Wire Read received NAK')
 
-    def select1WIRE(self, oneWireIndex):
+        return bytearray.fromhex(result[13:].replace("0x", ""))
 
-        self._sendCommand('1WIRE' + str(oneWireIndex) + ' SELECT')
-        result = self._readResponse()
-
-        return result
-
-    def skip1WIRE(self, oneWireIndex):
-
-        self._sendCommand('1WIRE' + str(oneWireIndex) + ' SKIP')
-        result = self._readResponse()
-
-        return result
-
-    def depower1WIRE(self, oneWireIndex):
-
-        self._sendCommand('1WIRE' + str(oneWireIndex) + ' DEPOWER')
-        result = self._readResponse()
-
-        return result
-
-    def getAddress1WIRE(self, oneWireIndex):
-
-        self._sendCommand('1WIRE' + str(oneWireIndex) + ' ADDR ?')
-        result = self._readResponse()
-
-        return result
-
-    def search1WIRE(self, oneWireIndex, normalSearch = True):
+    def search1WIRE(self, oneWireIndex=0, normalSearch=True):
+        """
+        Begin the process of searching for devices on the 1-WIRE bus.
+        The address of the found device will be stored in internal address buffer
 
+        :param oneWireIndex: The 1-Wire index, 0 on the binho nova
+        :type oneWireIndex: int
+        :param normalSearch: Whether to perform a normal search (returning all devices)
+                                or only return alarming devices
+        :type normalSearch: bool
+        :raises binhoException: if the command is unsuccessful
+        :return: None
+        :rtype: None
+        """
         if normalSearch:
-            self._sendCommand('1WIRE' + str(oneWireIndex) + ' SEARCH')
+            self._sendCommand(f'1WIRE{oneWireIndex} SEARCH')
         else:
-            self._sendCommand('1WIRE' + str(oneWireIndex) + ' SEARCH COND')
+            self._sendCommand(f'1WIRE{oneWireIndex} SEARCH COND')
 
-        result = self._readResponse()
+        if not self._checkDeviceSuccess(self._readResponse()):
+            raise binhoException('Error executing 1-Wire search received NAK')
 
-        return result
+    def resetSearch1WIRE(self, oneWireIndex=0):
+        """
+        Resets 1-Wire search so a new one can occur
 
-    def resetSearch1WIRE(self, oneWireIndex):
+        :param oneWireIndex: The 1-Wire index, 0 on the binho nova
+        :type oneWireIndex: int
+        :raises binhoException: if the command is unsuccessful
+        :return: None
+        :rtype: None
+        """
+        self._sendCommand(f'1WIRE{oneWireIndex} SEARCH RESET')
 
-        self._sendCommand('1WIRE' + str(oneWireIndex) + ' SEARCH RESET')
-        result = self._readResponse()
+        if not self._checkDeviceSuccess(self._readResponse()):
+            raise binhoException('Error executing 1-Wire search reset received NAK')
 
-        return result
+    def targetSearch1WIRE(self, target, oneWireIndex=0):
+        """
+        Searches the 1-WIRE bus for devices belonging to a specific family code
 
-    def targetSearch1WIRE(self, oneWireIndex, target):
+        :param oneWireIndex: The 1-Wire index, 0 on the binho nova
+        :type oneWireIndex: int
+        :param target: 1-byte long device family code for which to search
+        :type target: int
+        :raises binhoException: if the command is unsuccessful
+        :return: None
+        :rtype: None
+        """
+        if not 0 <= target <= 255:
+            raise binhoException(f'Target byte must be in range 0-255, not {target}')
 
-        self._sendCommand('1WIRE' + str(oneWireIndex) + ' SEARCH ' + str(target))
-        result = self._readResponse()
+        self._sendCommand(f'1WIRE{oneWireIndex} SEARCH {target}')
 
-        return result
+        if not self._checkDeviceSuccess(self._readResponse()):
+            raise binhoException('Error executing 1-Wire target search received NAK')
 
     ## SWI COMMANDS
 
@@ -970,14 +1224,14 @@ class binhoHostAdapter:
         self._sendCommand('SWI' + str(swiIndex) + ' PACKET DATA ' + str(index) + ' ' + str(value))
         result = self._readResponse()
         
-        return result;
+        return result
 
     def setPacketDataFromBufferSWI(self, swiIndex, byteCount, bufferName):
 
         self._sendCommand('SWI' + str(swiIndex) + ' PACKET DATA ' + str(byteCount) + ' ' + str(bufferName))
-        result = self._readResponse();
+        result = self._readResponse()
 
-        return result;
+        return result
 
     def sendPacketSWI(self, swiIndex):
 
